﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace Example
{
	public partial class MainPage : UserControl
	{
		private const long _1KB = 1024;
		private const long _10KB = 10240;
		private const long _100KB = 102400;
		private const long _1MB = 1024 * 1024;
		private DateTime startTime = DateTime.Now;
		private bool uploadProcessing = false;
		private long speedLimit = 0;
		private DispatcherTimer timer;

		public MainPage()
		{
			InitializeComponent();
			this.Loaded += new RoutedEventHandler(MainPage_Loaded);
		}

		void MainPage_Loaded(object sender, RoutedEventArgs e)
		{
			this.timer = new DispatcherTimer() { Interval = new TimeSpan(0, 0, 0, 0, 333) };
			this.timer.Tick += this.TickOfLimitTimer;
		}

		private void RadUpload1_UploadStarted(object sender, Telerik.Windows.Controls.UploadStartedEventArgs e)
		{
			this.uploadProcessing = true;
			this.buttonCover.Visibility = Visibility.Visible;
			this.startTime = DateTime.Now;
			this.speedLimit = this.GetSpeedLimit();
			this.infoBuffer.Text = "Buffer size: " + this.RadUpload1.BufferSize + " B";
			this.timer.Start();
		}

		private void RadUpload1_UploadFinished(object sender, EventArgs e)
		{
			this.uploadProcessing = false;
			this.buttonCover.Visibility = Visibility.Collapsed;
			this.timer.Stop();
		}

		private void RadUpload1_ProgressChanged(object sender, EventArgs e)
		{
			this.ShowStatisticOnUpload();
			if (this.uploadProcessing &&
				this.speedLimit > 0 &&
				this.RadUpload1.CurrentSession.UploadedBytes > this.UploadLimit())
			{
				this.RadUpload1.PauseUpload();
				if (!this.timer.IsEnabled)
				{
					this.timer.Start();
				}
				System.Diagnostics.Debug.WriteLine("PauseUpload ====");
			}
		}

		private void TickOfLimitTimer(object sender, EventArgs e)
		{
			if (this.RadUpload1 != null &&
				this.RadUpload1.CurrentSession.IsPaused &&
				this.uploadProcessing &&
				this.speedLimit > 0 &&
				this.RadUpload1.CurrentSession.UploadedBytes < this.UploadLimit())
			{
				this.ShowStatisticAfterWait();
				if (this.uploadProcessing)
				{
					this.timer.Stop();
					this.RadUpload1.ResumeUpload();
					System.Diagnostics.Debug.WriteLine("---- ResumeUpload + timer.Stop");
				}
			}
		}

		private void speed_Click(object sender, RoutedEventArgs e)
		{
			this.RadUpload1.CancelUpload();
			long limit = this.GetSpeedLimit();
			long size = _100KB;
			if (limit < 0)
			{
				size = _100KB;
			}
			else if (limit <= _1KB)
			{
				size = _1KB;
			}
			else if (limit <= _10KB)
			{
				size = _10KB;
			}
			this.RadUpload1.BufferSize = (int)size;
		}

		private long GetSpeedLimit()
		{
			if (this.speed1KB != null && this.speed1KB.IsChecked == true)
			{
				return _1KB;
			}
			if (this.speed10KB != null && this.speed10KB.IsChecked == true)
			{
				return _10KB;
			}
			if (this.speed100KB != null && this.speed100KB.IsChecked == true)
			{
				return _100KB;
			}
			if (this.speed1MB != null && this.speed1MB.IsChecked == true)
			{
				return _1MB;
			}
			return -1;
		}
		private long UploadLimit()
		{
			TimeSpan ts = TimeSpan.FromTicks(DateTime.Now.Ticks - this.startTime.Ticks);
			long limit = (long)(this.speedLimit * ts.TotalSeconds);
			return limit;
		}

		private long UploadSpeed()
		{
			TimeSpan ts = TimeSpan.FromTicks(DateTime.Now.Ticks - this.startTime.Ticks);
			long speed = (long)(this.RadUpload1.CurrentSession.UploadedBytes / ts.TotalSeconds);
			return speed;
		}

		private void ShowStatisticOnUpload()
		{
			this.infoUploaded.Text = "Uploaded: " + this.RadUpload1.CurrentSession.UploadedBytes + " B";
			this.infoUploadLimit.Text = "Upload limit: " + this.UploadLimit() + " B";
			this.infoSpeedOnEnter.Text = "Speed on upload: " + this.UploadSpeed() + " B/s";
			System.Diagnostics.Debug.WriteLine(this.infoSpeedOnEnter.Text);
		}

		private void ShowStatisticAfterWait()
		{
			this.infoSpeedOnExit.Text = "Speed after wait: " + this.UploadSpeed() + " B/s";
			System.Diagnostics.Debug.WriteLine(this.infoSpeedOnExit.Text + "~~~~");
		}
	}
}
